#include "dice.h"
using namespace libgames;

Fixed_Roll::Fixed_Roll(int value)
{
	set(value);
}

int Fixed_Roll::roll() const
{
	return m_value;
}

void Fixed_Roll::set(int value)
{
	m_value = value;
}

Die::Die(int faces)
{
	set_faces(faces);
	m_prng = new mtwister;
	roll();
}

Die::Die(const Die & original)
{
	m_prng = new mtwister;
	*this = original;
}

Die::~Die()
{
	delete m_prng;
}

Die & Die::operator =(const Die & original)
{
	m_faces = original.m_faces;
	m_value = original.m_value;
	
	// Questionable, don't know for the moment if it is a good or a bad
	// thing but when you affect the value of a non-random object to
	// another one, trigerring the same operations on both lead them to be
	// in a identical state. It will also be the case for these dice.
	m_prng = new mtwister(original.m_prng->get_seed());

	return *this;
}

Die::operator int()
{
	return value();
}

int Die::roll() const
{
	m_value = m_prng->get_rand_range(1, m_faces);
	return m_value;
}

int Die::faces() const
{
	return m_faces;
}

void Die::set_faces(int faces)
{
	m_faces = faces;
}

int Reroll_Once_If_Top_Die::roll() const
{
	int value;
	value = Die::roll();
	if (value == m_faces) value += Die::roll();
	m_value = value;
	return m_value;
}

int Reroll_Forever_If_Top_Die::roll() const
{
	int value, roll;
	value = roll = Die::roll();
	while (roll == m_faces) value += roll = Die::roll();
	m_value = value;
	return m_value;
}

Algebric_Roll::Algebric_Roll(Die die, bool sign)
{
	m_die = new Die(die);
	m_sign = sign ? 1 : -1;
}

Algebric_Roll::Algebric_Roll(const Algebric_Roll & roll)
{
	m_die = new Die(*roll.m_die);
	m_sign = roll.m_sign;
}

Algebric_Roll::~Algebric_Roll()
{
	delete m_die;
}

Algebric_Roll & Algebric_Roll::operator =(const Algebric_Roll & roll)
{
	delete m_die;
	m_die = new Die(*roll.m_die);
	m_sign = roll.m_sign;
	return *this;
}

int Algebric_Roll::roll() const
{
	m_value = m_sign * m_die->roll();
	return m_value;
}

int Algebric_Roll::sign() const
{
	return m_sign;
}

void Dice::clear()
{
	while(m_rollable_scalars->size() >= 1) {
#ifdef DEBUG_THAT_F__ING_BUG
		cout << "size: " << m_rollable_scalars->size() << endl;
#endif
		//delete m_rollable_scalars->back();
		m_rollable_scalars->pop_back();
	}
}

Dice::Dice()
{
	m_value = 0;
	m_rollable_scalars = new roll_container;
}

Dice::~Dice()
{
	clear();
	delete m_rollable_scalars;
}

void Dice::append(const Rollable* addendum)
{
}

void Dice::append(const Rollable_Container * addendum)
{
	roll_container elements(addendum->elements());
	for (roll_container::iterator it = elements.begin(); it != elements.end(); it++) {
		this->append(*it);
	}
}

void Dice::append(const Rollable_Scalar * addendum)
{
	Rollable_Scalar * rollable = new Rollable_Scalar(*addendum);
	m_rollable_scalars->push_back(rollable);
}

void Dice::append(const int addendum)
{
	Fixed_Roll fixed(addendum);
	this->append(&fixed);
}

int Dice::roll() const
{
	int value = 0;
	for (roll_container::iterator it = m_rollable_scalars->begin(); it != m_rollable_scalars->end(); it++) {
		value += (*it)->roll();
	}
	m_value = value;
	return m_value;
}

roll_container Dice::elements() const
{
	return *m_rollable_scalars;
}
