#ifdef DEBUG_THAT_F__ING_BUG
#include <iostream>
using std::cout;
using std::endl;
#endif

#ifndef CONTAINER_TYPE
#define CONTAINER_TYPE 1
#endif

#if CONTAINER_TYPE == 1
#include <vector>
using std::vector;
#elif CONTAINER_TYPE == 2
#include <deque>
using std::deque;
#elif CONTAINER_TYPE == 3
#include <list>
using std::list;
#endif

#include <libcoyotl/mtwister.h>
using libcoyotl::mtwister;

namespace libgames
{
	class Rollable
	{
	
	protected:
		mutable int m_value;
	
	public:
		virtual int roll() const 
		{
			return 0;
		}
		virtual int value() const 
		{
			return m_value;
		}
	};

#if CONTAINER_TYPE == 1
	typedef vector<Rollable*> roll_container;
#elif CONTAINER_TYPE == 2
	typedef deque<Rollable*> roll_container;
#elif CONTAINER_TYPE == 3
	typedef list<Rollable*> roll_container;
#endif

	class Rollable_Scalar : public Rollable
	{
	
	public:
		virtual ~Rollable_Scalar() {}
	};

	class Rollable_Container : public Rollable
	{
	
	private:
		roll_container * m_elements;

	public:
		virtual ~Rollable_Container() {}
		virtual roll_container elements() const 
		{
			return *m_elements;
		}
	};

	class Fixed_Roll : public Rollable_Scalar
	{
	
	public:
		Fixed_Roll(int);

		int roll() const;

		void set(int);
	};

	class Die : public Rollable_Scalar
	{

	protected:
		int m_faces;
		mtwister* m_prng;

	public:
		Die(int = 6);
		Die(const Die &);
		virtual ~Die();
		Die & operator =(const Die &);

		operator int();

		int roll() const;

		int faces() const;
		void set_faces(int);
	};

	class Reroll_Once_If_Top_Die : public Die
	{

	public:
		int roll() const;
	};

	class Reroll_Forever_If_Top_Die : public Die
	{

	public:
		int roll() const;
	};

	class Algebric_Roll : public Rollable_Scalar
	{

	protected:
		Die* m_die;
		int m_sign;

	public:
		Algebric_Roll(Die, bool = true);
		Algebric_Roll(const Algebric_Roll &);
		virtual ~Algebric_Roll();
		Algebric_Roll & operator =(const Algebric_Roll &);

		int roll() const;

		int sign() const;
	};

	class Dice : public Rollable_Container
	{
	
	protected:
		roll_container * m_rollable_scalars;
		void clear();
	
	public:
		Dice();
		template<class T> Dice(const T &)
		{
			*this = original;
		}
		virtual ~Dice();
		template<class T> Dice operator =(const T &)
		{
			m_rollable_scalars.clear();
			this->append(value);
			return *this;
		}
		template<class T> Dice operator <<(const T & addendum)
		{
			append(&addendum);
			return *this;
		}
		void append(const Rollable*);
		void append(const Rollable_Container *);
		void append(const Rollable_Scalar *);
		void append(const int);

		int roll() const;

		roll_container elements() const;

		// Dice operator =(const string);
		// Dice operator <<(const Rollable &);
		// operator string();
	};
};
